# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2015 Nexedi SA and Contributors. All Rights Reserved.
#                    Ivan Tyagov <ivan@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from AccessControl import ClassSecurityInfo
from Products.ERP5Type import Permissions, PropertySheet
from Products.ERP5.Document.BigFile import BigFile

class DataStream(BigFile):
  """
  Represents a very big infinite file with a streaming API.
  Usually used to store raw data.
  """

  meta_type = 'ERP5 Data Stream'
  portal_type = 'Data Stream'
  add_permission = Permissions.AddPortalContent

  # Declarative security
  security = ClassSecurityInfo()
  security.declareObjectProtected(Permissions.AccessContentsInformation)

  # Declarative properties
  property_sheets = ( PropertySheet.CategoryCore
                    , PropertySheet.SortIndex
                    )

  def iterate(self, start_offset, end_offset):
    """
      Read chunks of data from a Data Stream and yield them.
    """
    data = self._baseGetData()
    for chunk in data.iterate(start_offset, end_offset - start_offset):
      yield chunk

  def readChunkList(self, start_offset, end_offset):
    """
      Read chunks of data from a Data Stream and return them.
    """
    chunk_list = []
    data = self._baseGetData()
    for chunk in data.iterate(start_offset, end_offset - start_offset):
      chunk_list.append(chunk)

    return chunk_list

  def appendData(self, data_chunk):
    db = self.getPortalObject().erp5_sql_connection()
    if self.getId() in ('150', '152', '154', '156', '989', '59'):
      # WARNING: This precaution is not enough because there can still be a
      #          race condition between a zope using the old code and another
      #          using the new code, when the table does not contain any data
      #          yet for this stream.
      if not db.query('SELECT 1 FROM data_stream_queue WHERE uid=%s'
                      % self.getUid(), max_rows=1)[1]:
        BigFile.appendData(self, data_chunk)
        return
    db.query('INSERT INTO data_stream_queue(uid, chunk) VALUES(%s,%s)'
             % (self.getUid(), db.string_literal(data_chunk)))

  def processDataStreamQueue(self):
    q = self.getPortalObject().erp5_sql_connection().query
    r = q('SELECT id, chunk FROM data_stream_queue WHERE uid=%s ORDER BY id'
          % self.getUid(), max_rows=0)[1]
    if r:
      self._appendData(''.join(x[1] for x in r))
      q('DELETE FROM data_stream_queue WHERE id IN (%s)'
        % ','.join(str(x[0]) for x in r))
